@@ -25,31 +25,25 @@
 **************************************************************************/
 
 
-#ifdef ROOT_EXEC
-	/*
-	** TCP port for the MSQL daemon
-	*/
-	
-#	define MSQL_PORT	1112
-	
-	/*
-	** UNIX Domain socket for MSQL daemon
-	*/
-	
-#	define MSQL_UNIX_ADDR	"/dev/msql"
-#else
-	/*
-	** TCP port for the MSQL daemon
-	*/
-	
-#	define MSQL_PORT	4333
-	
-	/*
-	** UNIX Domain socket for MSQL daemon
-	*/
-	
-#	define MSQL_UNIX_ADDR	"/tmp/msql.sock"
+/*
+** TCP port for the MSQL daemon
+*/
 
+#ifdef ROOT_EXEC
+#define MSQL_PORT	1112
+#else
+#define MSQL_PORT	4333
+#endif
+
+	
+/*
+** UNIX Domain socket for MSQL daemon
+*/
+
+#ifdef ROOT_EXEC
+#define MSQL_UNIX_ADDR	"/dev/msql"
+#else
+#define MSQL_UNIX_ADDR	"/tmp/msql.sock"
 #endif
 
 
