@@ -223,7 +223,7 @@ static closeServer(sock)
 **	Notes	: 
 */
 
-msqlClose(sock)
+void msqlClose(sock)
 	int	sock;
 {
 	char	buf[6];
@@ -313,7 +313,7 @@ static freeFieldList(fieldData)
 **	Notes	: 
 */
 
-msqlFreeResult(result)
+void msqlFreeResult(result)
 	m_result  *result;
 {
 	freeQueryData(result->queryData);
@@ -415,7 +415,8 @@ int msqlConnect(host)
 #ifdef HAVE_SYS_UN_H
 		/* Shouldn't get in here with UNIX socks */
 		strcpy(hostInfo,"Localhost via UNIX socket");
-		msqlDebug(MOD_API,"Server name = NULL.  Using UNIX sock\n");
+		msqlDebug(MOD_API,"Server name = NULL.  Using UNIX sock(%s)\n",
+			MSQL_UNIX_ADDR);
 		sock = socket(AF_UNIX,SOCK_STREAM,0);
 		if (sock < 0)
 		{
@@ -442,8 +443,8 @@ int msqlConnect(host)
 	else
 	{
 		sprintf(hostInfo,"%s via TCP/IP",host);
-		msqlDebug(MOD_API,"Server name = %s.  Using TCP sock\n",
-			host);
+		msqlDebug(MOD_API,"Server name = %s.  Using TCP sock (%d)\n",
+			host, MSQL_PORT);
 		sock = socket(AF_INET,SOCK_STREAM,0);
 		if (sock < 0)
 		{
@@ -485,6 +486,7 @@ int msqlConnect(host)
 		{
 			sprintf(msqlErrMsg,"Can't connect to MSQL server on %s",
 				host);
+			perror("Connect");
 			close(sock);
 			return(-1);
 		}
@@ -522,6 +524,7 @@ int msqlConnect(host)
 		{
 			strcpy(msqlErrMsg,"Unknown MSQL error");
 		}
+		closeServer(sock);
 		return(-1);
 	}
 
@@ -530,6 +533,7 @@ int msqlConnect(host)
 	if (!cp)
 	{
 		strcpy(msqlErrMsg,"Bad packet received from server");
+		closeServer(sock);
 		return(-1);
 	}
 	version = atoi(cp + 1);
@@ -538,6 +542,7 @@ int msqlConnect(host)
 		sprintf(msqlErrMsg,
 		   "Protocol mismatch. Server Version = %d Client Version = %d",
 			version, PROTOCOL_VERSION);
+		closeServer(sock);
 		return(-1);
 	}
 	msqlDebug(MOD_API,"mSQL protocol version - API=%d, server=%d\n",
@@ -598,9 +603,9 @@ int msqlConnect(host)
 		{
 			strcpy(msqlErrMsg,"Unknown MSQL error");
 		}
+		closeServer(sock);
 		return(-1);
 	}
-
 	return(sock);
 }
 
