@@ -45,6 +45,13 @@
 #include "y.tab.h"
 
 #define	REG		register
+#define NUM_HASH	16
+
+#ifndef DEBUG
+#  define malloc(s) 	Malloc(s,__FILE__,__LINE__)
+#  define free(a) 	Free(a,__FILE__,__LINE__)
+#endif
+
 
 /*
 ** Macros for handling the scanner's internal pointers
@@ -112,48 +119,97 @@ typedef struct symtab_s {
 } symtab_t;
 
 
-static symtab_t symtab [] = {
-	{ "create",	token(CREATE)},
-	{ "drop",	token(DROP)},
-	{ "insert",	token(INSERT)},
-	{ "delete",	token(DELETE)},
-	{ "select",	token(SELECT)},
-	{ "update",	token(UPDATE)},
-	{ "all",	token(ALL)},
-	{ "distinct",	token(DISTINCT)},
-	{ "where",	token(WHERE)},
-	{ "order",	token(ORDER)},
-	{ "from",	token(FROM)},
-	{ "into",	token(INTO)},
-	{ "table",	token(TABLE)},
-	{ "by",		token(BY)},
-	{ "asc",	token(ASC)},
-	{ "desc",	token(DESC)},
-	{ "like",	token(LIKE)},
-	{ "and",	token(AND)},
-	{ "or",		token(OR)},
-	{ "values",	token(VALUES)},
-	{ "set",	token(SET)},
-	{ "not",	token(NOT)},
-	{ "null",	token(NULLSYM)},
-	{ "primary",	token(PRIMARY)},
-	{ "key",	token(KEY)},
-	{ "int",	token(INT)},
-	{ "bool",	token(BOOL)},
-	{ "char",	token(CHAR)},
-	{ "real",	token(REAL)},
-	{ ">=",		token(GE)},
-	{ "<=",		token(LE)},
-	{ "<>",		token(NE)},
-	{ "=",		token(EQ)},
-	{ ">",		token(GT)},
-	{ "<",		token(LT)},
-	{ "\0",		0}
+static symtab_t symtab[16][16] = {
+	{ /* 0 */
+		{ "select",	token(SELECT)},
+		{ "values",	token(VALUES)},
+		{ 0,		0}
+	},
+	{ /* 1 */
+		{ "or",		token(OR)},
+		{ "not",	token(NOT)},
+		{ 0,		0}
+	},
+	{ /* 2 */
+		{ "distinct",	token(DISTINCT)},
+		{ 0,		0}
+	},
+	{ /* 3 */
+		{ "and",	token(AND)},
+		{ "delete",	token(DELETE)},
+		{ "update",	token(UPDATE)},
+		{ 0,		0}
+	},
+	{ /* 4 */
+		{ "from",	token(FROM)},
+		{ "create",	token(CREATE)},
+		{ "primary",	token(PRIMARY)},
+		{ "smallint",	token(INT)},
+		{ "real",	token(REAL)},
+		{ 0,		0}
+	},
+	{ /* 5 */
+		{ "drop",	token(DROP)},
+		{ "insert",	token(INSERT)},
+		{ "like",	token(LIKE)},
+		{ 0,		0}
+	},
+	{ /* 6 */
+		{ 0,		0}
+	},
+	{ /* 7 */
+		{ "asc",	token(ASC)},
+		{ 0,		0}
+	},
+	{ /* 8 */
+		{ "table",	token(TABLE)},
+		{ 0,		0}
+	},
+	{ /* 9 */
+		{ "<=",		token(LE)},
+		{ "all",	token(ALL)},
+		{ "key",	token(KEY)},
+		{ 0,		0}
+	},
+	{ /* 10 */
+		{ "<>",         token(NE)},
+		{ "into",	token(INTO)},
+		{ 0,		0}
+	},
+	{ /* 11 */
+		{ "where",	token(WHERE)},
+		{ ">=",		token(GE)},
+		{ "by",		token(BY)},
+		{ "null",	token(NULLSYM)},
+		{ "int",	token(INT)},
+		{ 0,		0}
+	},
+	{ /* 12 */
+		{ "<",		token(LT)},
+		{ "order",	token(ORDER)},
+		{ "set",	token(SET)},
+		{ 0,		0}
+	},
+	{ /* 13 */
+		{ "=",          token(EQ)},
+		{ 0,		0}
+	},
+	{ /* 14 */
+		{ ">",          token(GT)},
+		{ "integer",	token(INT)},
+		{ "char",	token(CHAR)},
+		{ 0,		0}
+	},
+	{ /* 15 */
+		{ "desc",	token(DESC)},
+		{ 0,		0}
+	}
 };
 
 
 
 
+
 msqlInitScanner(buf)
 	u_char	*buf;
 {
@@ -168,12 +224,23 @@ int findKeyword(tok,len)
 	int	len;
 {
 	REG	char	*cp1,
-			*cp2;
+			*cp2,
+			tmp;
 	REG	symtab_t *stab;
 	int	found;
+        REG 	int     hash=0,
+			index=0;
 
-	stab = symtab;
-	while(*(stab->name))
+
+        cp1 = tok;
+        while(*cp1 && index++ < len)
+        {
+                hash += *cp1++;
+        }
+        hash = hash & (NUM_HASH - 1);
+
+	stab = symtab[hash];
+	while(stab->name)
 	{
 		cp1 = stab->name;
 		cp2 = tok;
@@ -185,7 +252,13 @@ int findKeyword(tok,len)
 				found = 0;
 				break;
 			}
+/*
 			if (tolower(*cp2++) != *cp1++)
+*/
+			tmp = *cp2++;
+			if (tmp >64 && tmp<91)
+				tmp+=32;
+			if (tmp != *cp1++)
 			{
 				found = 0;
 				break;
