@@ -85,8 +85,8 @@ typedef struct field_ps {
 ** Where clause list element 
 */
 typedef struct cond_s {			
-	char	table[NAME_LEN],
-		name[NAME_LEN];
+	char	table[NAME_LEN + 1],
+		name[NAME_LEN + 1];
 	val_t 	*value;
 	int	op,
 		bool,
@@ -98,7 +98,7 @@ typedef struct cond_s {
 
 
 typedef struct tlist_s {
-	char	name[NAME_LEN];
+	char	name[NAME_LEN + 1];
 	struct	tlist_s *next;
 } tlist_t;
 
@@ -107,8 +107,8 @@ typedef struct tlist_s {
 ** Order clause list element  (not used as yet!)
 */
 typedef struct order_s {		
-	char	table[NAME_LEN],
-		name[NAME_LEN];
+	char	table[NAME_LEN + 1],
+		name[NAME_LEN + 1];
 	int	dir,
 		type,
 		length;
@@ -122,8 +122,9 @@ typedef struct order_s {
 ** Table cache entry struct 
 */
 typedef struct cache_s {		
-	char	DB[NAME_LEN],
-		table[NAME_LEN],
+	char	DB[NAME_LEN + 1],
+		table[NAME_LEN + 1],
+		resInfo[4 * NAME_LEN], /* used for debugging info */
 		*rowBuf,
 		*keyBuf;
 	int	age,
@@ -137,11 +138,14 @@ typedef struct cache_s {
 
 #ifdef	HAVE_MMAP
 	int	remapData,
-		remapKey;
+		remapKey,
+		remapStack;
 	caddr_t	dataMap,
-		keyMap;
+		keyMap,
+		stackMap;
 	off_t	size,
-		keySize;
+		keySize,
+		stackSize;
 #else
 	char	readBuf[BUF_SIZE];
         u_int   firstRow, 
@@ -243,8 +247,11 @@ typedef char	* C_PTR;
 #define RELOAD_ACL	9
 #define SHUTDOWN	10
 
-#define NEED_REMAP	1
-#define NO_REMAP	2
+#define NO_REMAP	0
+#define DATA_REMAP	1
+#define KEY_REMAP	2
+#define STACK_REMAP	4
+#define FULL_REMAP	0xffff
 
 #define DEST_CLIENT	1
 #define DEST_TABLE	2
@@ -260,5 +267,25 @@ static char *comTable[] = {
 	"Shutdown", "???" };
 
 
+#define MALLOC_BLK      1
+#define MMAP_BLK        2
+ 
+static char *blockTags[] = {
+        "ERROR : Unknown block tag",
+        "malloc()",
+        "mmap()",
+        "ERROR : Unknown block tag"
+};
+ 
+
+
+
 void writePkt();
 int readPkt();
+
+
+#ifndef _MSQL_SERVER_SOURCE
+
+extern	char	*msqlHomeDir;
+
+#endif
