@@ -61,10 +61,11 @@ static	cond_t	*condTail = NULL;
 static	field_t	*fieldTail = NULL;
 static	order_t	*orderTail = NULL;
 static	tlist_t	*tableTail = NULL;
+static 	int	havePriKey = 0;
 
 
-#define	malloc(L)	Malloc(L)
-#define	free(A)		Free(A)
+#define	malloc(L)	Malloc(L,__FILE__,__LINE__)
+#define	free(A)		Free(A,__FILE__,__LINE__)
 #define	safeFree(x)	{if (x) { (void) free(x); x = NULL; }}
 #define REG		register
 
@@ -89,8 +90,9 @@ msqlClean()
 	register order_t 	*curOrder, *tmpOrder;
 	register tlist_t	*curTable, *tmpTable;
 
-	dTrace(TRACE_IN,"msqlClean()");
+	msqlTrace(TRACE_IN,"msqlClean()");
 	command = 0;
+	havePriKey = 0;
 
 	/*
 	** blow away the table list from the query
@@ -149,7 +151,7 @@ msqlClean()
 
 	msqlBackendClean();
 
-	dTrace(TRACE_OUT,"msqlClean()");
+	msqlTrace(TRACE_OUT,"msqlClean()");
 }
 
 
@@ -160,8 +162,30 @@ ident_t *msqlCreateIdent(seg1,seg2)
 {
 	ident_t	*new;
 
+	msqlTrace(TRACE_IN,"msqlCreateIdent()");
+	if (seg1)
+	{
+		if (strlen(seg1) > NAME_LEN)
+		{
+			sprintf(packet,
+				"-1:Identifier name '%s' too long\n",seg1);
+			writePkt(outSock);
+			msqlTrace(TRACE_OUT,"msqlCreateIdent()");
+			return(NULL);
+		}
+	}
+	if (seg2)
+	{
+		if (strlen(seg2) > NAME_LEN)
+		{
+			sprintf(packet,
+				"-1:Identifier name '%s' too long\n",seg2);
+			writePkt(outSock);
+			msqlTrace(TRACE_OUT,"msqlCreateIdent()");
+			return(NULL);
+		}
+	}
 	new = (ident_t *)malloc(sizeof(ident_t));
-	(void)bzero(new,sizeof(ident_t));
 	if (seg1)
 	{
 		(void)strcpy(new->seg1,seg1);
@@ -170,6 +194,7 @@ ident_t *msqlCreateIdent(seg1,seg2)
 	{
 		(void)strcpy(new->seg2,seg2);
 	}
+	msqlTrace(TRACE_OUT,"msqlCreateIdent()");
 	return(new);
 }
 
@@ -204,7 +229,6 @@ static u_char expandEscape(c,remain)
 
 
 
-
 val_t *msqlCreateValue(textRep,type,tokLen)
 	u_char	*textRep;
 	int	type,
@@ -218,29 +242,15 @@ val_t *msqlCreateValue(textRep,type,tokLen)
 			*cp2;
 	static	u_char nullVal[] = "null";
 
+	msqlTrace(TRACE_IN,"msqlCreateValue()");
 	new = (val_t *)malloc(sizeof(val_t));
-	(void)bzero(new,sizeof(val_t));
 	new->type = type;
 	new->dataLen = tokLen;
-	cp = nullVal;
-	cp2 = textRep;
-	match = 1;
-	while(*cp)
-	{
-		if (*cp != tolower(*cp2))
-		{
-			match = 0;
-			break;
-		}
-		cp++;
-	}
-	if (match && *cp2 == 0)
-	{
-		new->nullVal = 1;
-		return(new);
-	}
 	switch(type)
 	{
+		case NULL_TYPE:
+			new->nullVal = 1;
+			break;
 		case IDENT_TYPE:
 			new->val.identVal = (ident_t *)textRep;
 			break;
@@ -279,6 +289,7 @@ val_t *msqlCreateValue(textRep,type,tokLen)
 			sscanf((char *)textRep ,"%lf",&new->val.realVal);
 			break;
 	}
+	msqlTrace(TRACE_OUT,"msqlCreateValue()");
 	return(new);
 }
 
@@ -290,8 +301,8 @@ val_t *fillValue(val,type,length)
 {
 	val_t	*new;
 
+	msqlTrace(TRACE_IN,"fillValue()");
 	new = (val_t *)malloc(sizeof(val_t));
-	(void)bzero(new,sizeof(val_t));
 	new->type = type;
 	new->nullVal = 0;
 	switch(type)
@@ -302,13 +313,18 @@ val_t *fillValue(val,type,length)
 			break;
 
 		case INT_TYPE:
+#ifndef _CRAY
 			new->val.intVal = (int) * (int *)val;
+#else
+			new->val.intVal = unpackInt32(val);
+#endif
 			break;
 
 		case REAL_TYPE:
 			new->val.realVal = (double) * (double *)val;
 			break;
 	}
+	msqlTrace(TRACE_OUT,"fillValue()");
 	return(new);
 }
 
@@ -318,7 +334,6 @@ val_t *nullValue()
 	val_t	*new;
 
 	new = (val_t *)malloc(sizeof(val_t));
-	(void)bzero(new,sizeof(val_t));
 	new->nullVal = 1;
 	return(new);
 }
@@ -328,18 +343,24 @@ val_t *nullValue()
 freeValue(val)
 	val_t	*val;
 {
+	msqlTrace(TRACE_IN,"freeValue()");
 	if (!val)
+	{
+		msqlTrace(TRACE_OUT,"freeValue()");
 		return;
+	}
 	switch(val->type)
 	{
 		case IDENT_TYPE:
 			(void)free(val->val.identVal);
 			break;
 		case CHAR_TYPE:
-			(void)free(val->val.charVal);
+			if (!val->nullVal)
+				(void)free(val->val.charVal);
 			break;
 	}
 	(void)free(val);
+	msqlTrace(TRACE_OUT,"freeValue()");
 }
 
 
@@ -356,7 +377,7 @@ freeValue(val)
 **		  if the field is of type CHAR
 */
 
-msqlAddField(ident,type,length,notNull,priKey)
+int msqlAddField(ident,type,length,notNull,priKey)
 	ident_t	*ident;
 	int 	type;
 	char	*length;
@@ -367,7 +388,24 @@ msqlAddField(ident,type,length,notNull,priKey)
 	char	*name,
 		*table;
 
-	dTrace(TRACE_IN,"msqlAddField()");
+	msqlTrace(TRACE_IN,"msqlAddField()");
+
+	if (priKey)
+	{
+		if (havePriKey)
+		{
+			sprintf(packet,"-1:Can't have multiple primary keys\n");
+			writePkt(outSock);
+			msqlTrace(TRACE_OUT,"msqlAddField()");
+			return(-1);
+
+		}
+		else
+		{
+			havePriKey = 1;
+		}
+	}
+
 
 	if (*(ident->seg2))
 	{
@@ -380,12 +418,11 @@ msqlAddField(ident,type,length,notNull,priKey)
 		table = NULL;
 	}
 	new = (field_t *)malloc(sizeof(field_t));
-	(void)bzero(new,sizeof(field_t));
 	if (table)
 	{
-		(void)strcpy(new->table,table);
+		(void)strncpy(new->table,table,NAME_LEN - 1);
 	}
-	(void)strcpy(new->name,name);
+	(void)strncpy(new->name,name,NAME_LEN - 1);
 	if (notNull)
 	{
 		new->flags |= NOT_NULL_FLAG;
@@ -426,7 +463,9 @@ msqlAddField(ident,type,length,notNull,priKey)
 		fieldTail->next = new;
 		fieldTail = new;
 	}
-	dTrace(TRACE_OUT,"msqlAddField()");
+	free(ident);
+	msqlTrace(TRACE_OUT,"msqlAddField()");
+	return(0);
 }
 
 
@@ -436,6 +475,7 @@ msqlAddFieldValue(value)
 	register field_t	*fieldVal;
 	u_char	*buf;
 
+	msqlTrace(TRACE_IN,"msqlAddFieldValue()");
 	if (!lastField)
 	{
 		lastField = fieldVal = fieldHead;
@@ -456,6 +496,7 @@ msqlAddFieldValue(value)
 		}
 		fieldVal->value = value;
 	}
+	msqlTrace(TRACE_OUT,"msqlAddFieldValue()");
 }
 
 
@@ -481,7 +522,7 @@ msqlAddCond(ident,op,value,bool)
 	char	*name,
 		*table;
 
-	dTrace(TRACE_IN,"msqlAddCond()");
+	msqlTrace(TRACE_IN,"msqlAddCond()");
 
 	if (*(ident->seg2))
 	{
@@ -495,7 +536,6 @@ msqlAddCond(ident,op,value,bool)
 	}
 
 	new = (cond_t *)malloc(sizeof(cond_t));
-	(void)bzero(new,sizeof(cond_t));
 	(void)strcpy(new->name,name);
 	if (table)
 	{
@@ -515,7 +555,8 @@ msqlAddCond(ident,op,value,bool)
 		condTail->next = new;
 		condTail = new;
 	}
-	dTrace(TRACE_OUT,"msqlAddCond()");
+	free(ident);
+	msqlTrace(TRACE_OUT,"msqlAddCond()");
 }
 
 
@@ -535,10 +576,9 @@ msqlAddOrder(ident,dir)
 {
 	register order_t	*new;
 
-	dTrace(TRACE_IN,"msqlAddOrder()");
+	msqlTrace(TRACE_IN,"msqlAddOrder()");
 
 	new = (order_t *)malloc(sizeof(order_t));
-	(void)bzero(new,sizeof(order_t));
 	if (*ident->seg1)
 	{
 		(void)strcpy(new->table,ident->seg1);
@@ -554,7 +594,8 @@ msqlAddOrder(ident,dir)
 		orderTail->next = new;
 		orderTail = new;
 	}
-	dTrace(TRACE_OUT,"msqlAddOrder()");
+	free(ident);
+	msqlTrace(TRACE_OUT,"msqlAddOrder()");
 }
 
 
@@ -565,10 +606,9 @@ msqlAddTable(name)
 {
 	register tlist_t	*new;
 
-	dTrace(TRACE_IN,"msqlAddTable()");
+	msqlTrace(TRACE_IN,"msqlAddTable()");
 
 	new = (tlist_t *)malloc(sizeof(tlist_t));
-	(void)bzero(new,sizeof(tlist_t));
 	(void)strcpy(new->name,name);
 	(void)free(name);
 	if (!tableHead)
@@ -580,7 +620,7 @@ msqlAddTable(name)
 		tableTail->next = new;
 		tableTail = new;
 	}
-	dTrace(TRACE_OUT,"msqlAddTable()");
+	msqlTrace(TRACE_OUT,"msqlAddTable()");
 }
 
 
@@ -614,13 +654,13 @@ msqlProcessQuery()
 {
 	int	res;
 
-	dTrace(TRACE_IN,"msqlProcessQuery()");
+	msqlTrace(TRACE_IN,"msqlProcessQuery()");
 	if (!curDB)
 	{
 		sprintf(packet,"-1:No current database\n");
 		writePkt(outSock);
-		debug(MOD_ERR,"No current database\n");
-		dTrace(TRACE_OUT,"msqlProcessQuery()");
+		msqlDebug(MOD_ERR,"No current database\n");
+		msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 		return;
 	}
 	switch(command)
@@ -631,7 +671,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlSelect(tableHead,fieldHead,condHead,
@@ -643,7 +683,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlCreate(tableHead->name,fieldHead,curDB);
@@ -654,7 +694,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlUpdate(tableHead->name,fieldHead,condHead,
@@ -666,7 +706,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlInsert(tableHead->name,fieldHead,curDB);
@@ -677,7 +717,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlDelete(tableHead->name,condHead,curDB);
@@ -688,7 +728,7 @@ msqlProcessQuery()
 			{
 				sprintf(packet,"-1:Access Denied\n");
 				writePkt(outSock);
-				dTrace(TRACE_OUT,"msqlProcessQuery()");
+				msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 				return;
 			}
 			res = msqlDrop(tableHead->name,curDB);
@@ -700,10 +740,10 @@ msqlProcessQuery()
 		char	errBuf[180];
 
 		sprintf(packet,"-1:%s\n",errMsg);
-		dTrace(TRACE_OUT,"msqlProcessQuery()");
+		msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 		writePkt(outSock);
 	}
-	dTrace(TRACE_OUT,"msqlProcessQuery()");
+	msqlTrace(TRACE_OUT,"msqlProcessQuery()");
 }
 
 
@@ -711,9 +751,10 @@ msqlQueryOverrunError(txt)
 	char	*txt;
 {
 
+	msqlTrace(TRACE_IN,"msqlQueryOverrunError()");
 	sprintf(packet,"-1:Syntax error.  Bad text after query. '%s'\n",txt);
-	dTrace(TRACE_OUT,"msqlProcessQuery()");
 	writePkt(outSock);
+	msqlTrace(TRACE_OUT,"msqlQueryOverrunError()");
 }
 
 
@@ -723,7 +764,10 @@ char *msqlParseQuery(inBuf,sock)
         char    *inBuf;
         int     sock;
 {
+	msqlTrace(TRACE_IN,"msqlParseQuery()");
         outSock = sock;
         msqlInitScanner(inBuf);
         yyparse();
+	msqlTrace(TRACE_OUT,"msqlParseQuery()");
 }
+
