@@ -66,7 +66,9 @@ int	curSock,
 	UNIXsock,
 	numCons;
 
-cinfo_t	conArray[MAX_CON + 3];  /* leave room for stdout etc if debugging */
+#define MAX_SOCK_NUM	100
+
+cinfo_t	conArray[MAX_SOCK_NUM];
 
 extern	char	*packet;
 char	PROGNAME[] = "msqld",
@@ -83,7 +85,7 @@ void sigTrap(sig)
 	fprintf(stderr,"\nHit by a sig %d\n\n",sig);
 	clientSock = 3;
 	printf("\n\nForced server shutdown due to bad signal!\n\n");
-	while(clientSock < MAX_CON)
+	while(clientSock < MAX_SOCK_NUM)
 	{
 		if (conArray[clientSock].db)
 		{
@@ -150,6 +152,7 @@ initServer()
 	/*
 	** Create an IP socket
 	*/
+	msqlDebug(MOD_GENERAL,"IP Socket is %d\n",MSQL_PORT);
 	IPsock = socket(AF_INET, SOCK_STREAM, 0);
 	if (IPsock < 0)
 	{
@@ -171,6 +174,7 @@ initServer()
 	/*
 	** Create the UNIX socket
 	*/
+	msqlDebug(MOD_GENERAL,"UNIX Socket is %s\n",MSQL_UNIX_ADDR);
 	UNIXsock = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (UNIXsock < 0)
 	{
@@ -206,7 +210,7 @@ RETSIGTYPE puntServer(sig)
 	{
 		printf("\n\nServer Aborting!\n\n");
 	}
-	while(clientSock < MAX_CON)
+	while(clientSock < MAX_SOCK_NUM)
 	{
 		if (conArray[clientSock].db)
 		{
@@ -225,6 +229,19 @@ RETSIGTYPE puntServer(sig)
 	unlink(MSQL_UNIX_ADDR);
 #endif
 	printf("\n");
+	dropCache();
+
+	if (debugSet(MOD_MALLOC))
+	{
+		fprintf(stderr,"\n\nmalloc() leak detection .....\n");
+		checkBlocks(MALLOC_BLK);
+	}
+	if (debugSet(MOD_MMAP))
+	{
+		fprintf(stderr,"\n\nmmap() leak detection .....\n");
+		checkBlocks(MMAP_BLK);
+	}
+	printf("\n\nmSQL Daemon Shutdown Complete.\n\n");
 	if (sig >= 0)
 	{
 		exit(1);
@@ -236,7 +253,7 @@ RETSIGTYPE puntServer(sig)
 yyerror(s)
         char	*s;
 {
- 	sprintf(packet,"-1:%s near \"%s\"\n", s, yytext);
+ 	sprintf(packet,"-1:%s near \"%s\"\n", s, yytext?yytext:"");
 	writePkt(curSock);
 	msqlClean();
 }
@@ -380,9 +397,14 @@ main(argc,argv)
 	/*
 	** We have enough space for fiddling with the argv, continue
 	*/
+	msqlHomeDir = (char *)getenv("MSQL_HOME");
+	if (!msqlHomeDir)
+	{
+		msqlHomeDir = INST_DIR;
+	}
 	umask(0);
 	numCons=0;
-	chdir(INST_DIR);
+	chdir(msqlHomeDir);
 	initDebug();
 	initNet();
 	initServer();
@@ -399,11 +421,12 @@ main(argc,argv)
 		fprintf(pidFile,"%d",getpid());
 		fclose(pidFile);
 	}
+	chmod(path,0744);
+	umask(0);
 	setupSignals();
-	msqlLoadAcl();
+	msqlLoadAcl(1);
 	(void)bzero(&clientFDs,sizeof(fd_set));
-	msqlBackendInit();
-	debug(MOD_GENERAL,"miniSQL debug mode.  Waiting for connections.\n");
+	msqlDebug(MOD_GENERAL,"miniSQL debug mode.  Waiting for connections.\n");
 	while(1)
 	{
 		(void)bzero(&readFDs,sizeof(fd_set));
@@ -413,7 +436,7 @@ main(argc,argv)
 		FD_SET(UNIXsock,&readFDs);
 #endif
 		setProcTitle("Idle - %d clients",numCons);
-		if(select(MAX_CON+1,&readFDs,0,0,0) < 0)
+		if(select(MAX_SOCK_NUM+1,&readFDs,0,0,0) < 0)
 			continue;
 
 		/*
@@ -452,10 +475,26 @@ main(argc,argv)
 				conArray[newSock].db = NULL;
 			}
 
+			/*
+			** Are we over the connection limit
+			*/
+			numCons++;
+			if (numCons > MAX_CON)
+			{
+				numCons--;
+				sendError(newSock,"Too many connections");
+				shutdown(newSock,2);
+				close(newSock);
+				continue;
+			}
+
+
 			/*
 			** store the connection details
 			*/
 
+			msqlDebug(MOD_GENERAL,"New connection received on %d\n",
+				newSock);
 			error = 0;
 			if (sock == IPsock)
 			{
@@ -485,7 +524,7 @@ main(argc,argv)
 				{
 					conArray[newSock].host = (char *)
 						strdup(hp->h_name);
-					debug(MOD_GENERAL,"Host = %s\n",
+					msqlDebug(MOD_GENERAL,"Host = %s\n",
 						hp->h_name);
 				}
 			}
@@ -496,7 +535,7 @@ main(argc,argv)
 					sizeof(struct sockaddr));
 				bzero(&conArray[newSock].
 					remote,sizeof(struct sockaddr));
-				debug(MOD_GENERAL,"Local connection\n");
+				msqlDebug(MOD_GENERAL,"Host = UNIX domain\n");
 			}
 
 			if (!error)
@@ -513,18 +552,23 @@ main(argc,argv)
 					sendError(newSock,"Bad handshake");
 					shutdown(newSock,2);
 					close(newSock);
+					conArray[newSock].host = NULL;
+                                	bzero(&conArray[newSock].local,
+                                        	sizeof(struct sockaddr));
+                                	bzero(&conArray[newSock].
+                                        	remote,sizeof(struct sockaddr));
+
 				}
 				else
 				{
 					FD_SET(newSock,&clientFDs);
 					uname = (char *)strtok(packet,"\n");
-					debug(MOD_GENERAL,"User = %s\n",uname);
+					msqlDebug(MOD_GENERAL,"User = %s\n",uname);
 					conArray[newSock].user = (char *)
 						strdup(uname);
 					sprintf(packet,"-100:\n");
 					writePkt(newSock);
 				}
-				numCons++;
 			}
 			continue;
 		}
@@ -536,7 +580,7 @@ main(argc,argv)
 		*/	
 
 		comSock = 3;
-		while(comSock < MAX_CON)
+		while(comSock < MAX_SOCK_NUM)
 		{
 		    if (FD_ISSET(comSock,&readFDs))
 		    {
@@ -545,17 +589,17 @@ main(argc,argv)
 				command = QUIT;
 			else
 				command = atoi(packet);
-			debug(MOD_GENERAL,"Command = %d (%s)\n",command,
-				comTable[command]);
+			msqlDebug(MOD_GENERAL,"Command on sock %d = %d (%s)\n",
+				comSock, command, comTable[command]);
 			switch(command)
 			{
 			    case INIT_DB:
 				setProcTitle("Init DB");
 				cp=(char *)strtok(packet+2,"\n\r");
 				strcpy(dbname,cp);
-				debug(MOD_GENERAL,"DBName = %s\n", dbname);
+				msqlDebug(MOD_GENERAL,"DBName = %s\n", dbname);
 				conArray[comSock].access = msqlCheckAccess(
-					dbname, &conArray[comSock]);
+					dbname, conArray + comSock);
 				if(conArray[comSock].access == NO_ACCESS)
 				{
 					sendError(comSock,
@@ -585,7 +629,9 @@ main(argc,argv)
 				curSock = comSock;
 				cp=(char *)(packet+2);
 				arg = (char *)strdup(cp);
-				debug(MOD_QUERY,"Query = %s\n",arg);
+				if (debugSet(MOD_QUERY))
+					fprintf(stderr,"\n");
+				msqlDebug(MOD_QUERY,"Query = %s",arg);
 				msqlSetDB(conArray[comSock].db);
 				msqlSetPerms(conArray[comSock].access);
 				msqlParseQuery(arg,comSock);
@@ -627,7 +673,7 @@ main(argc,argv)
 				break;
 
 			    case QUIT:
-				debug(MOD_GENERAL,"DB QUIT!\n");
+				msqlDebug(MOD_GENERAL,"DB QUIT!\n");
 				FD_CLR(comSock,&clientFDs);
 				shutdown(comSock,2);
 				close(comSock);
@@ -635,16 +681,15 @@ main(argc,argv)
 				{
 				   safeFree(conArray[comSock].db);
 				   safeFree(conArray[comSock].host);
-				   safeFree(conArray[newSock].user);
+				   safeFree(conArray[comSock].user);
 				}
 				conArray[comSock].db = NULL;
 				numCons--;
 				break;
 		
-			    case CREATE_DB:	
+			    case CREATE_DB:
 				setProcTitle("Create DB");
-				if (conArray[comSock].host || 
-				   strcmp(conArray[comSock].user,ROOT)!=0)
+				if (!msqlCheckLocal(conArray + comSock))
 				{
 					sendError(comSock,"Permission denied");
 					break;
@@ -657,8 +702,7 @@ main(argc,argv)
 
 			    case DROP_DB:	
 				setProcTitle("Drop DB");
-				if (conArray[comSock].host || 
-				   strcmp(conArray[comSock].user,ROOT)!=0)
+				if (!msqlCheckLocal(conArray + comSock))
 				{
 					sendError(comSock,"Permission denied");
 					break;
@@ -671,8 +715,7 @@ main(argc,argv)
 
 			    case RELOAD_ACL:
 				setProcTitle("Reload ACL");
-				if (conArray[comSock].host || 
-				   strcmp(conArray[comSock].user,ROOT)!=0)
+				if (!msqlCheckLocal(conArray + comSock))
 				{
 					sendError(comSock,"Permission denied");
 					break;
@@ -684,8 +727,7 @@ main(argc,argv)
 
 			    case SHUTDOWN:
 				setProcTitle("Shutdown");
-				if (conArray[comSock].host || 
-				   strcmp(conArray[comSock].user,ROOT)!=0)
+				if (!msqlCheckLocal(conArray + comSock))
 				{
 					sendError(comSock,"Permission denied");
 					break;
@@ -701,7 +743,7 @@ main(argc,argv)
 					"Unknown command");
 				break;
 			}
-			debug(MOD_GENERAL,"Command Processed!\n");
+			msqlDebug(MOD_GENERAL,"Command Processed!\n");
 		    }
 		    comSock++;
 		}
